@@ -7,12 +7,11 @@ use Types::Standard qw(Maybe ArrayRef InstanceOf HashRef Bool);
 use Carp qw(croak);
 use Scalar::Util qw(blessed);
 
-use Form::Tiny::PathValue;
 use Form::Tiny::Error;
-use Form::Tiny::Utils qw(try get_package_form_meta);
+use Form::Tiny::Utils qw(try);
 use Moo::Role;
 
-our $VERSION = '2.07';
+our $VERSION = '2.08';
 
 has 'field_defs' => (
 	is => 'ro',
@@ -84,7 +83,7 @@ sub _ft_mangle_field
 {
 	my ($self, $def, $path_value, $out_ref) = @_;
 
-	my $current = $out_ref ? $path_value : $path_value->value;
+	my $current = $out_ref ? $path_value : $path_value->{value};
 
 	# We got the parameter, now we have to check if it is not empty
 	# Even if it is, it may still be handled if isn't hard-required
@@ -100,7 +99,7 @@ sub _ft_mangle_field
 			$$out_ref = $current;
 		}
 		else {
-			$path_value->set_value($current);
+			$path_value->{value} = $current;
 		}
 
 		return 1;
@@ -112,38 +111,41 @@ sub _ft_mangle_field
 sub _ft_find_field
 {
 	my ($self, $fields, $field_def) = @_;
+	my @path = @{$field_def->get_name_path->path};
+	my @meta = @{$field_def->get_name_path->meta};
 
 	# the result goes here
 	my @found;
 	my $traverser;
 	$traverser = sub {
-		my ($curr_path, $path, $index, $value) = @_;
-		my $last = $index == @{$path->meta};
+		my ($curr_path, $index, $value) = @_;
 
-		if ($last) {
+		if ($index == @meta) {
+
+			# we reached the end of the tree
 			push @found, [$curr_path, $value];
 		}
 		else {
-			my $next = $path->path->[$index];
-			my $meta = $path->meta->[$index];
+			my $next = $path[$index];
+			my $meta = $meta[$index];
 
 			if ($meta eq 'ARRAY' && ref $value eq 'ARRAY') {
 				for my $ind (0 .. $#$value) {
 					return    # may be an error, exit early
-						unless $traverser->([@$curr_path, $ind], $path, $index + 1, $value->[$ind]);
+						unless $traverser->([@$curr_path, $ind], $index + 1, $value->[$ind]);
 				}
 
 				if (@$value == 0) {
 
 					# we wanted to have a deeper structure, but its not there, so clearly an error
-					return unless $index == $#{$path->meta};
+					return unless $index == $#meta;
 
 					# we had aref here, so we want it back in resulting hash
 					push @found, [$curr_path, [], 1];
 				}
 			}
 			elsif ($meta eq 'HASH' && ref $value eq 'HASH' && exists $value->{$next}) {
-				return $traverser->([@$curr_path, $next], $path, $index + 1, $value->{$next});
+				return $traverser->([@$curr_path, $next], $index + 1, $value->{$next});
 			}
 			else {
 				# something's wrong with the input here - does not match the spec
@@ -154,14 +156,14 @@ sub _ft_find_field
 		return 1;    # all ok
 	};
 
-	if ($traverser->([], $field_def->get_name_path, 0, $fields)) {
+	if ($traverser->([], 0, $fields)) {
 		return [
 			map {
-				Form::Tiny::PathValue->new(
+				{
 					path => $_->[0],
 					value => $_->[1],
 					structure => $_->[2]
-				)
+				}
 			} @found
 		];
 	}
@@ -170,23 +172,25 @@ sub _ft_find_field
 
 sub _ft_assign_field
 {
-	my ($self, $fields, $field_def, $path_value) = @_;
+	my ($self, $fields, $field_def, $path_values) = @_;
 
 	my @arrays = map { $_ eq 'ARRAY' } @{$field_def->get_name_path->meta};
-	my @parts = @{$path_value->path};
-	my $current = \$fields;
-	for my $i (0 .. $#parts) {
+	for my $path_value (@$path_values) {
+		my @parts = @{$path_value->{path}};
+		my $current = \$fields;
+		for my $i (0 .. $#parts) {
 
-		# array_path will contain array indexes for each array marker
-		if ($arrays[$i]) {
-			$current = \${$current}->[$parts[$i]];
-		}
-		else {
-			$current = \${$current}->{$parts[$i]};
+			# array_path will contain array indexes for each array marker
+			if ($arrays[$i]) {
+				$current = \${$current}->[$parts[$i]];
+			}
+			else {
+				$current = \${$current}->{$parts[$i]};
+			}
 		}
+
+		$$current = $path_value->{value};
 	}
-
-	$$current = $path_value->value;
 }
 
 ### OPTIMIZATION: detect and use faster route for flat forms
@@ -234,17 +238,20 @@ sub _ft_validate_nested
 		my $current_data = $self->_ft_find_field($fields, $validator);
 		if (defined $current_data) {
 			my $all_ok = 1;
+			my @to_assign;
 
 			# This may have multiple iterations only if there's an array
 			foreach my $path_value (@$current_data) {
-				unless ($path_value->structure) {
-					$path_value->set_value($inline_hook->($self, $validator, $path_value->value))
+				unless ($path_value->{structure}) {
+					$path_value->{value} = ($inline_hook->($self, $validator, $path_value->{value}))
 						if $inline_hook;
 					$all_ok = $self->_ft_mangle_field($validator, $path_value) && $all_ok;
 				}
-				$self->_ft_assign_field($dirty, $validator, $path_value);
+				push @to_assign, $path_value;
 			}
 
+			$self->_ft_assign_field($dirty, $validator, \@to_assign);
+
 			# found and valid, go to the next field
 			next if $all_ok;
 		}
@@ -254,10 +261,12 @@ sub _ft_validate_nested
 			$self->_ft_assign_field(
 				$dirty,
 				$validator,
-				Form::Tiny::PathValue->new(
-					path => $validator->get_name_path->path,
-					value => $validator->get_default($self),
-				)
+				[
+					{
+						path => $validator->get_name_path->path,
+						value => $validator->get_default($self),
+					}
+				]
 			);
 		}
 		elsif ($validator->required) {
@@ -385,14 +394,6 @@ sub _ft_clear_errors
 	return;
 }
 
-# This fixes form inheritance for other role systems than Moose
-around DOES => sub {
-	my ($orig, $self, @args) = @_;
-
-	return Moo::Role::does_role($self, @args)
-		|| $self->$orig(@args);
-};
-
 1;
 
 __END__
