@@ -17,7 +17,7 @@ require Moo::Role;
 
 use namespace::clean;
 
-our $VERSION = '2.07';
+our $VERSION = '2.08';
 
 # more clear error messages in some crucial cases
 our @CARP_NOT = qw(Form::Tiny Form::Tiny::Form);
@@ -116,11 +116,9 @@ sub run_hooks_for
 {
 	my ($self, $stage, @data) = @_;
 
-	my @hooks = @{$self->hooks->{$stage} // []};
-
 	# running hooks always returns the last element they're passed
 	# (unless they are not modifying, then they don't return anything)
-	for my $hook (@hooks) {
+	for my $hook (@{$self->hooks->{$stage} // []}) {
 		my $ret = $hook->code->(@data);
 		splice @data, -1, 1, $ret
 			if $hook->is_modifying;
@@ -135,7 +133,7 @@ sub _inline_hook
 
 	my @hooks = @{$self->hooks->{$stage} // []};
 
-	return if @hooks == 0;
+	return undef if @hooks == 0;
 	return sub {
 		my @data = @_;
 
@@ -152,10 +150,11 @@ sub _inline_hook
 sub bootstrap
 {
 	my ($self) = @_;
-	return unless !$self->complete;
+	return if $self->complete;
 
 	# package name may be non-existent if meta is anon
 	if ($self->has_package) {
+
 		# when this breaks, mst gets to point and laugh at me
 		my @parents = do {
 			my $package_name = $self->package;
@@ -190,6 +189,8 @@ sub resolved_fields
 {
 	my ($self, $object) = @_;
 
+	return [@{$self->fields}] if $self->is_flat;
+
 	croak 'resolved_fields requires form object'
 		unless defined blessed $object;
 
